'''
Created on Oct 24, 2009
http://stackoverflow.com/questions/311703/algorithm-for-sampling-without-replacement
@author: johnsalvatier
'''

from __future__ import division
import numpy 

def random_no_replace(sampleSize, populationSize, numSamples):
    
    samples  = numpy.zeros((numSamples, sampleSize),dtype=int)
    
    # Use Knuth's variable names
    cdef int n = sampleSize
    cdef int N = populationSize

    cdef i = 0
    cdef int t = 0 # total input records dealt with
    cdef int m = 0 # number of items selected so far
    cdef double u

    while i < numSamples:

        t = 0
        m = 0 
        while m < n :

            
            u = numpy.random.uniform() # call a uniform(0,1) random number generator

            if  (N - t)*u >= n - m :
            
                t += 1
            
            else:
            
                samples[i,m] = t
                t += 1
                m += 1
                
        i += 1
        
    return samples
    




"""
Created on Oct 24, 2009

@author: johnsalvatier

Introduction
------------ 


Most of the sampler logic takes place in DreamSampler.sample(). DreamSampler contains many MultiChain objects which represent one 
chain, each DreamChain object contains a MultiChainStepper object. The DreamSampler object does the looping and coordinates the step methods
Most of the logic in MultiChain and MultiChainStepper is just pass through logic.
"""
from __future__ import division
from pymc import *

from numpy import *

from convergence import GRConvergence, CovarianceConvergence
from multichain import MultiChainSampler, MultiChain
import time 
import dream_components
from simulation_history import *


class DreamSampler(MultiChainSampler):
    """
    Implements a variant of DREAM_ZS using PyMC. The sampler is a multi-chain sampler that proposal states based on the differences between 
    random past states. The sampler does not use the snooker updater but does use the crossover probability, probability distribution. Convergence
    assessment is based on a naive implementation of the Gelman-Rubin convergence statistics.
 
    The basis for this algorithm are the following papers:
     
        Provides the basis for the DREAM_ZS extension (also see second paper).
        C.J.F. ter Braak, and J.A. Vrugt, Differential evolution Markov chain with
        snooker updater and fewer chains, Statistics and Computing, 18(4),
        435-446, doi:10.1007/s11222-008-9104-9, 2008.    
        
        Introduces the origional DREAM idea:
        J.A. Vrugt, C.J.F. ter Braak, C.G.H. Diks, D. Higdon, B.A. Robinson, and
        J.M. Hyman, Accelerating Markov chain Monte Carlo simulation by
        differential evolution with self-adaptive randomized subspace sampling,
        International Journal of Nonlinear Sciences and Numerical
        Simulation, 10(3), 273-290, 2009.
        
        This paper uses DREAM in an application
        J.A. Vrugt, C.J.F. ter Braak, M.P. Clark, J.M. Hyman, and B.A. Robinson,
        Treatment of input uncertainty in hydrologic modeling: Doing hydrology
        backward with Markov chain Monte Carlo simulation, Water Resources
        Research, 44, W00B09, doi:10.1029/2007WR006720, 2008.
    """
    
    _numbers = None
    acceptRatio = 0.0
    
    def sample(self, ndraw = 1000, ndraw_max = 20000 , nChains = 5, burnIn = 100, thin = 5, convergenceCriteria = 1.1,variables_of_interest = None,  nCR = 3, DEpairs = 1, adaptationRate = .65, eps = 5e-6, mConvergence = False, mAccept = False):
        """
        Samples from a posterior distribution using DREAM.
        
        Parameters
        ----------
        ndraw : int 
            minimum number of draws from the sample distribution to be returned 
        ndraw_max : int 
            maximum number of draws from the sample distribution to be returned
        nChains : int 
            number of different chains to employ
        burnInSize : int
            number of iterations (meaning draws / nChains) to do before doing actual sampling.
        nCR : int
            number of intervals to use to adjust the crossover probability distribution for efficiency
        DEpairs : int 
            number of pairs of chains to base movements off of
        eps : float
            used in jittering the chains
            
        Returns
        -------
            None : None 
                sample sets 
                self.history which contains the combined draws for all the chains
                self.iter which is the total number of iterations 
                self.acceptRatio which is the acceptance ratio
                self.burnIn which is the number of burn in iterations done 
                self.R  which is the gelman rubin convergence diagnostic for each dimension
        """
        
        startTime = time.time()
        
        maxChainDraws = floor(ndraw_max/nChains)     
        
        self._initChains(nChains, ndraw_max)    
        
        history = SimulationHistory(maxChainDraws, self._nChains, self.dimensions)
        
        if variables_of_interest is not None:
            slices = []
            for var in variables_of_interest:
                slices.append(self.slices[var])
        else:
            slices = [slice(None,None)]
        history.add_group('interest', slices)
        
        # initialize the temporary storage vectors
        currentVectors = zeros((nChains, self.dimensions))
        currentLogPs = zeros(nChains)
        
        
        #) make a list of starting chains that at least spans the dimension space
        # in this case it will be of size 2*dim
        nSeedChains = int(ceil(self.dimensions* 2/nChains) * nChains)
        nSeedIterations = int(nSeedChains/nChains) 

        
        model = self._model_generator()
        for i in range(nSeedIterations - 1): 
            vectors =  zeros((nChains, self.dimensions))
            for j in range(nChains):
            
                #generate a vector drawn from the prior distributions
                for variable in model:
                    if isinstance(variable,Stochastic) and not variable.observed:
                        drawFromPrior = variable.random()
                        if isinstance(drawFromPrior , np.matrix):
                            drawFromPrior = drawFromPrior.A.ravel()
                        elif isinstance(drawFromPrior, np.ndarray):
                            drawFromPrior = drawFromPrior.ravel()
                        else:
                            drawFromPrior = drawFromPrior
                        
                        vectors[j,self.slices[str(variable)]] = drawFromPrior
            
            history.record(vectors,0,0)
 
        #use the last nChains chains as the actual chains to track
        vectors =  self._vectors

        
        #add the starting positions to the history
        history.record(vectors,self._logPs,0)

        gamma = None       
               
                
        # initilize the convergence diagnostic object
        grConvergence = GRConvergence()
        covConvergence = CovarianceConvergence()

        
        # get the starting log likelihood and position for each of the chains 
        currentVectors = vectors
        currentLogPs = self._logPs
        
        
        #2)now loop through and sample 
        
        minDrawIters = ceil(ndraw / nChains)
        maxIters = ceil(ndraw_max /nChains)
        
        
        iter = 0
        accepts_ratio_weighting = 1 - exp(-1.0/30) 
       
        lastRecalculation = 0
        
        # continue sampling if:
        # 1) we have not drawn enough samples to satisfy the minimum number of iterations
        # 2) or any of the dimensions have not converged 
        # 3) and we have not done more than the maximum number of iterations 

        while ( history.nsamples < ndraw or any(grConvergence.R > convergenceCriteria)) and history.ncombined_history < ndraw_max:
            
            if iter  == burnIn:
                history.start_sampling()
                
            #every5th iteration allow a big jump
            if random.randint(5) == 0.0:
                gamma = array([1.0])
            else:
                gamma = array([2.38 / sqrt( 2 * DEpairs  * self.dimensions)])

            proposalVectors = dream_components.dream_proposals(currentVectors, history,self.dimensions, nChains, DEpairs, gamma, .05, eps)

            
            # get the log likelihoods for the proposal chains 
            self._propose(proposalVectors)
            proposalLogPs = self._logPs

                    
            #apply the metrop decision to decide whether to accept or reject each chain proposal        
            decisions, acceptance = self._metropolis_hastings(currentLogPs,proposalLogPs) 
                

            self._update_accepts_ratio(accepts_ratio_weighting, acceptance)
            if mAccept and iter % 20 == 0:
                print self.accepts_ratio 
            
            self._reject(decisions)
    
            #make the current vectors the previous vectors 
            previousVectors = currentVectors
            currentVectors = choose(decisions[:,newaxis], (currentVectors, proposalVectors))

            currentLogPs = choose(decisions, (currentLogPs, proposalLogPs))
            
                    
            # we only want to recalculate convergence criteria when we are past the burn in period
            if history.nsamples > 0 and iter > lastRecalculation * 1.1 and history.nsequence_histories > self.dimensions:

                lastRecalculation = iter
                grConvergence.update(history)
                covConvergence.update(history,'all')
                covConvergence.update(history,'interest')
                
                if mConvergence:
                    print mean(grConvergence.R), std(grConvergence.R), max(grConvergence.R), argmax(grConvergence.R)
                    print covConvergence.relativeVariances['interest']

            if iter % thin == 0:
                
                historyStartMovementRate = adaptationRate
                #try to adapt more when the acceptance rate is low and less when it is high 
                if adaptationRate == 'auto':
                    historyStartMovementRate = min((.234/self.accepts_ratio)*.5, .95)
                    
                history.record(currentVectors, currentLogPs, historyStartMovementRate)

                    
                
            iter += 1

            
            
        
        #3) finalize
        
        # only make the second half of draws available because that's the only part used by the convergence diagnostic
        
        self.history = history.samples
        self.iter = iter

        self.burnIn = burnIn 
        self.time = time.time() - startTime
        
        self.R = grConvergence.R
        
        self._finalizeChains()
            
    


import timeit 


t = timeit.Timer ("from numpy import single, float_, longfloat; float_dtypes = [float, single, float_, longfloat]; single in float_dtypes and float_ in float_dtypes")
print t.timeit(50000)
from numpy import *

ones("Sd")




'''
Created on Jan 20, 2010

@author: johnsalvatier
'''

from numpy import *
from pymc import *
import pymc

class testClass:
    
    __array_priority__ = 1000
    

        
    def __index__(self):
        print "index"
        
    def __rmul__(self, other):
        print "rmul"
        return 1
  
# this is the way that pymc does this   
def __getitem__(self, index):
    print "getitem", index   
    
testClass.__getitem__ = types.UnboundMethodType(__getitem__, None, testClass)

x = array([[1,2,3,7],[4,5,1,0]])
y = testClass()


a = x * y


'''
Created on Jan 13, 2010

@author: johnsalvatier

implementation of adaptive Metropolist adjusted Langevin algorithm (Adaptive MaLa) mcmc sampling technique

'''


from __future__ import division
from pymc import *

from numpy import *
from convergence import GRConvergence
from multichain import MultiChainSampler, MultiChain
import time 


class HamiltonianSampler(MultiChainSampler):
    """

    """
    
    acceptRatio = 0.0
    
    def sample(self, ndraw = 1000, adaptationConstant = .90,alpha = .5, steps = 10, debug = Inf, ndraw_max = 20000 , nChains = 5, burnIn = 100, thin = 1, convergenceCriteria = 1.1, mConvergence = False, mAccept = False):
        """
        Samples from a posterior distribution using DREAM.
        
        Parameters
        ----------
        ndraw : int 
            minimum number of draws from the sample distribution to be returned 
        ndraw_max : int 
            maximum number of draws from the sample distribution to be returned
        nChains : int 
            number of different chains to employ
        burnInSize : int
            number of iterations (meaning draws / nChains) to do before doing actual sampling.
            
        Returns
        -------
            None : None 
                sample sets 
                self.history which contains the combined draws for all the chains
                self.iter which is the total number of iterations 
                self.acceptRatio which is the acceptance ratio
                self.burnIn which is the number of burn in iterations done 
                self.R  which is the gelman rubin convergence diagnostic for each dimension
        """
        startTime = time.time()
        
        maxChainDraws = floor(ndraw_max/nChains)     
        
        self._initChains(nChains, ndraw_max)   
        
        # get the starting log likelihood and position for each of the chains 
        currentVectors =random.normal(loc = 1.0, scale = .01, size = (self._nChains, self.dimensions)) #self._vectors'
        currentVectors[:,0] -= 1.0
        self._propose(currentVectors)
        
        currentLogPs = self._logPs
        currentGradLogPs = self._gradLogPs        
        
        #initialize the history arrays   
        combinedHistory = zeros((nChains * maxChainDraws , self.dimensions))
        sequenceHistories = zeros((nChains, self.dimensions, maxChainDraws))
        logPSequences = zeros((nChains, maxChainDraws))
        
        #add the starting positions to the history
        sequenceHistories[:,:,0] = currentVectors
        combinedHistory[0:nChains,:] = currentVectors    
        logPSequences[:, 0] = currentLogPs
        

                          
        # initilize the convergence diagnostic object
        grConvergence = GRConvergence()
              

        
        #2)now loop through and sample 
        
        minDrawIters = ceil(ndraw / nChains)
        maxIters = ceil(ndraw_max /nChains)
        
        
        iter = 1
    
        acceptsRatio = 0
        relevantHistoryStart = 0
        relevantHistoryEnd = 1 
       
        lastRecalculation = 0
        
        adaptedMean = 0
        adaptedCov = 1.0
        
        # continue sampling if:
        # 1) we have not drawn enough samples to satisfy the minimum number of iterations
        # 2) or any of the dimensions have not converged 
        # 3) and we have not done more than the maximum number of iterations 
        momentumVectors = 0 
        
        while ( relevantHistoryStart < burnIn or (relevantHistoryEnd - relevantHistoryStart) *nChains  < ndraw or any(grConvergence.R > convergenceCriteria)) and  relevantHistoryEnd*nChains  < ndraw_max:

            adaptedMean, adaptedCov = self._adapt(currentVectors,adaptationConstant/max(relevantHistoryStart-burnIn, 1), adaptedMean, adaptedCov) 
            
            # generate momentum vectors
            
            momentumVectors = momentumVectors * alpha + .5 * (1- alpha**2) * random.normal(size = (self._nChains, self.dimensions))
            
            stepMomentum = momentumVectors 
            stepGradLogPs = currentGradLogPs
            scale = diagonal(adaptedCov)
            if iter % debug == 0 :
                print "c", currentVectors
                print "cM", momentumVectors
                print "cp", currentLogPs
                print "cg", currentGradLogPs
                
            
            for i in range(steps):
                
                halfStepMomentum = stepMomentum + (.5/steps) * stepGradLogPs
                stepVectors = currentVectors + (.5/steps) *  halfStepMomentum * scale
                
                self._propose(stepVectors)
                stepGradLogPs = self._gradLogPs
                
                stepMomentum = halfStepMomentum + (.5/steps) * stepGradLogPs
                if ( iter %debug == 0):
                    print i
                    print "s", stepVectors
                    print "sM", stepMomentum
                    print "sg", stepGradLogPs  
            
            proposalVectors = stepVectors
            proposalLogPs = self._logPs
            proposalGradLogPs = stepGradLogPs
                
            #apply the metrop decision to decide whether to accept or reject each chain proposal        
            decisions, acceptance = self._metropolis_hastings(currentLogPs + sum(momentumVectors**2 / (2 )* scale, axis = 1),
                                                              proposalLogPs+ sum(stepMomentum**2 / (2 )* scale, axis = 1)) 
                
                
            
            weighting = 1 - exp(-1.0/60) 
            acceptsRatio = weighting * sum(acceptance)/nChains + (1-weighting) * acceptsRatio
            if mAccept and iter % 20 == 0:
                print acceptsRatio 


            #make the current vectors the previous vectors 
            previousVectors = currentVectors
            
            currentVectors = choose(decisions[:,newaxis], (currentVectors, proposalVectors))
            currentLogPs = choose(decisions, (currentLogPs, proposalLogPs))
            currentGradLogPs = choose(decisions[:, newaxis], (currentGradLogPs, proposalGradLogPs))
            
            #need to repropose these because some of these may have been rolled back more than 1 "proposal"
            self._propose(currentVectors)
                    
            # we only want to recalculate convergence criteria when we are past the burn in period
            # and then only every so often (currently every 10% increase in iterations)
            if (relevantHistoryStart > burnIn  and
                (relevantHistoryEnd - relevantHistoryStart) * nChains > ndraw and  
                iter > lastRecalculation * 1.1):

                lastRecalculation = iter
                # calculate the Gelman Rubin convergence diagnostic 
                grConvergence.update(sequenceHistories, relevantHistoryEnd, relevantHistoryStart, self.dimensions, nChains)
                if mConvergence:
                    print mean(grConvergence.R), std(grConvergence.R), max(grConvergence.R), argmax(grConvergence.R)

            #record the vector for each chain
            if iter % thin == 0:
                sequenceHistories[:,:,relevantHistoryEnd] = currentVectors
                combinedHistory[(relevantHistoryEnd *nChains) :(relevantHistoryEnd *nChains + nChains),:] = currentVectors
                logPSequences[:, relevantHistoryEnd] = currentLogPs
                
                relevantHistoryEnd += 1
                relevantHistoryStart += .5
                      
            iter += 1
   
        
        #3) finalize
        
        # only make the second half of draws available because that's the only part used by the convergence diagnostic
        
        self.history = combinedHistory[relevantHistoryStart*nChains:relevantHistoryEnd*nChains,:]
        self.iter = iter
        self.acceptRatio = acceptsRatio 
        self.burnIn = burnIn 
        self.time = time.time() - startTime
        
        self.R = grConvergence.R
        
        self._finalizeChains()
        
        
    def _adapt(self, currentVectors,adaptationRate, oldMean, oldCov):
        
        newMean = oldMean + adaptationRate *  mean(currentVectors - oldMean, axis = 0)
        newCov = oldCov + adaptationRate * (dot((currentVectors - oldMean).transpose(),currentVectors - oldMean)/self._nChains - oldCov) 

        return newMean, newCov


'''
Created on Jan 20, 2010

@author: johnsalvatier
'''

from numpy import *
from pymc import *
import pymc
from scipy.stats import distributions as d

print "start"
nPredictors = 20
observations = 100

def rmul2 (self, other):
    print "rmul2"
    return 0

observedPredictors = d.norm(loc = 0, scale = 1.0).rvs((nPredictors,observations))

responses = pymc.Normal("responses", mu = zeros((nPredictors,1)), tau = ones((nPredictors,1)) * 5**-2 )

print 1
print responses.__rmul__
responses.__rmul__ = rmul2
o =  observedPredictors * responses


'''
Created on Nov 24, 2009

@author: johnsalvatier
'''
import model7

import pymc 
import multichain_mcmc
from pylab import *
import numpy 

sampler = multichain_mcmc.DreamSampler(model7.model_gen)
sampler.sample(nChains = 10, ndraw = 1000, adaptationRate = 'auto', mAccept = True, mConvergence = True)
slices = sampler.slices

print sampler.R
history = sampler.history
print history.shape
samples = history.shape[0]
print sampler.accepts_ratio
print sampler.burnIn

meanInterceptEstimate = numpy.mean(history[:,slices['intercept']], axis = 0)
meanSdEstimate = numpy.mean(history[:,slices['sd']], axis = 0)
meanResponsesEstimates = numpy.mean(history[:,slices['responses']], axis = 0)


print meanInterceptEstimate, meanSdEstimate
print meanInterceptEstimate/model7.trueIntercept, meanSdEstimate/model7.trueErrorSd
print meanResponsesEstimates
print meanResponsesEstimates/model7.trueResponses


subplot(3,3,1)
hist(history[:, 0])
subplot(3,3,2)
hist(history[:, 1])
subplot(3,3,3)
hist(history[:, 2])
subplot(3,3,4)
hist(history[:, 3])
subplot(3,3,5)
plot(history[:, 4])
subplot(3,3,6)
plot(history[:, 5])
subplot(3,3,7)
plot(history[:, 1])
subplot(3,3,8)
plot(history[:, 2])
subplot(3,3,9)
plot(history[:, 3])

show()





import model8

import pymc 
import multichain_mcmc
import numpy 
import time
from pylab import *

start = time.time()
sampler = multichain_mcmc.AmalaSampler(model8.model_gen)
sampler.sample(ndraw = 500,  maxGradient = 1.3, mAccept = True, mConvergence = True)
print  (time.time() - start)    

history = sampler.history
samples = sampler.samples
slices = sampler.slices

print history
subplot(1,2,1)
plot(history[:, slices['mu']])
subplot(1,2,2)
hist(samples[:, slices['mu']])
show()


from numpy import *
import pymc
from scipy import stats
from scipy.stats import distributions as d

#parameters about the da
nPredictors = 20
observations = 100

trueIntercept = d.norm(loc = 0, scale = 20).rvs(1)
trueResponses = d.norm(loc = 0, scale = 2).rvs(nPredictors)
trueErrorSd = d.gamma.rvs(5, scale = .05, size = 1)

observedPredictors = d.norm(loc = 0, scale = 1.0).rvs((nPredictors,observations))


data = trueIntercept + sum(observedPredictors * trueResponses[:,newaxis], axis = 0) +  d.norm(  loc = 0, scale = trueErrorSd).rvs(observations)

def model_gen():
    
    variables = []
    
    intercept = pymc.Normal("intercept",mu = 0, tau = 50**-2)
    sd = pymc.Gamma("sd", alpha = 1 , beta = .5)

    responses = pymc.Normal("responses", mu = zeros((nPredictors,1)), tau = ones((nPredictors,1)) * 5**-2 )
    
    variables.append(intercept)
    variables.append(responses)
    variables.append (sd)
    
    
    #@pymc.deterministic
    #def obsMeans (intercept = intercept, responses = responses):
    #    return intercept + sum(observedPredictors * responses[:,newaxis], axis = 0) 
    obsMeans = intercept + observedPredictors * responses
    
    obs = pymc.Normal ("obs", mu = obsMeans, tau = sd**-2, observed = True, value = data)
    
    variables.append(obs)
    
    return variables


'''
Created on Jan 27, 2010

@author: johnsalvatier
'''
from numpy import array, newaxis, random, sum, arange, floor
from rand_no_replace import random_no_replace


def dream_proposals( currentVectors, history, dimensions, nChains, DEpairs, gamma, jitter, eps ):
    """
    generates and returns proposal vectors given the current states
    """
    
    sampleRange = history.ncombined_history
    currentIndex = arange(sampleRange - nChains,sampleRange)[:, newaxis]
    combined_history = history.combined_history

    #choose some chains without replacement to combine
    chains = random_no_replace(DEpairs * 2, sampleRange - 1, nChains)
    
    # makes sure we have already selected the current chain so it is not replaced
    # this ensures that the the two chosen chains cannot be the same as the chain for which the jump is
    chains += (chains >= currentIndex)
    
    chainDifferences =  (sum(combined_history[chains[:, 0:DEpairs], :], axis = 1)      - 
                         sum(combined_history[chains[:, DEpairs:(DEpairs*2)], :], axis = 1))
    
    e = random.normal(0, jitter, (nChains,dimensions))

    E = random.normal(0, eps,(nChains,dimensions)) # could replace eps with 1e-6 here

    proposalVectors = currentVectors + (1 + e) * gamma[:,newaxis] * chainDifferences + E
    return proposalVectors




def dream2_proposals( currentVectors, history, dimensions, nChains, DEpairs, gamma, jitter, eps ):
    """
    generates and returns proposal vectors given the current states
    """
    
    sampleRange = history.ncombined_history
    currentIndex = arange(sampleRange - nChains,sampleRange)[:, newaxis]
    combined_history = history.combined_history

    #choose some chains without replacement to combine
    chains = random_no_replace(1, sampleRange - 1, nChains)
    
    # makes sure we have already selected the current chain so it is not replaced
    # this ensures that the the two chosen chains cannot be the same as the chain for which the jump is
    chains += (chains >= currentIndex)
    

    proposalVectors = combined_history[chains[:, 0], :]
    return proposalVectors



'''
Created on Nov 25, 2009

@author: johnsalvatier
'''

from numpy import *
import pymc
from scipy import stats
import pylab 
zeroStart = stats.distributions.norm(mu = 0, scale = 200).rvs(1)
ReData = arange(200, 5000, 10)
maxRe = max(ReData)
measured = 10.2 * (ReData )** .5 + stats.distributions.norm(mu = 0, scale = 55).rvs(len(ReData)) + zeroStart
#pylab.plot(ReData, measured)
#pylab.show()

otherData = stats.distributions.norm(mu = zeroStart, scale = 30).rvs(10)

def model_gen():
    
    varlist = []
    
    stdev = pymc.TruncatedNormal('stdev', mu=400, tau = 1.0/(400**2), a = 0, b = Inf)
    varlist.append(stdev)
    
    
    @pymc.deterministic
    def precision (stdev = stdev):
        return 1.0/(stdev**2)
    

    
    
    fakeA = pymc.TruncatedNormal('a', mu = 1, tau = 1.0/(50**2),a = 0, b = Inf)
    b = pymc.Uniform('b', lower = .05, upper = 2.0)
    a = fakeA * maxRe ** (-b)
    z = pymc.Normal('zero',mu = 0, tau = 1.0/(400**2))
    
    varlist.append(fakeA)
    varlist.append(a)
    varlist.append(b)
    varlist.append(z)


    @pymc.deterministic
    def nonlinear (Re = ReData,
                   value = measured,
                   a = a, b = b , z  = z,
                   observed = True
                   ):
        return (a * (ReData )** b) + z
    
    results = pymc.Normal('results', mu = nonlinear, tau = precision, value = measured, observed = True)
    varlist.append(results)
    
    return varlist


'''
Created on Nov 24, 2009

@author: johnsalvatier
'''
import model4

import pymc 
import multichain_mcmc
from pylab import *
import numpy 
#NOT A WORKING EXAMPLE
sampler = multichain_mcmc.DreamSampler(model4.model_gen)
sampler.sample(nChains = 10,adaptationRate = 'auto', mConvergence = True,mAccept = True)
slices = sampler.slices

print sampler.R
history = sampler.history
print history.shape
samples = history.shape[0]
print sampler.acceptRatio
print sampler.burnIn

subplot(3,3,1)
hist(history[:, 0])
subplot(3,3,2)
hist(history[:, 1])
subplot(3,3,3)
hist(history[:, 2])
subplot(3,3,3)
hist(history[:, 3])
subplot(3,3,5)
plot(history[:, 0])
subplot(3,3,6)
plot(history[:, 1])
subplot(3,3,8)
plot(history[:, 2])
subplot(3,3,9)
plot(history[:, 3])


print model4.trueFactorMagnitudes
print model4.trueFactorLoadings
print model4.trueErrorSds

meanLoadingEstimates = numpy.mean(history[:,slices['factorloadings']], axis = 0)
meanSdEstimates = numpy.mean(history[:,slices['residualsds']], axis = 0)
print meanSdEstimates 
print meanLoadingEstimates/model4.trueFactorLoadings
print meanSdEstimates/model4.trueErrorSds



show()



'''
Created on Nov 24, 2009

@author: johnsalvatier
'''
import model7

import pymc 
import multichain_mcmc
from pylab import *
import numpy 

variables_of_interest = ['mu']

sampler = multichain_mcmc.AmalaSampler(model7.model_gen)
sampler.sample(ndraw = 500,samplesPerAdapatationParameter = .5,adaptationDecayLength = 100, variables_of_interest = variables_of_interest, ndraw_max = 10000, maxGradient = 100)
slices = sampler.slices

print sampler.R
history = sampler.samples
print history.shape
samples = history.shape[0]
print sampler.accepts_ratio
print sampler.burnIn
print "running time (s)", sampler.time 

meanInterceptEstimate = numpy.mean(history[:,slices['intercept']], axis = 0)
meanSdEstimate = numpy.mean(history[:,slices['sd']], axis = 0)
meanResponsesEstimates = numpy.mean(history[:,slices['responses']], axis = 0)


print meanInterceptEstimate, meanSdEstimate
print meanInterceptEstimate/model7.trueIntercept, meanSdEstimate/model7.trueErrorSd
print meanResponsesEstimates
print meanResponsesEstimates/model7.trueResponses


subplot(3,3,1)
hist(history[:, slices['intercept']])
subplot(3,3,2)
hist(history[:, slices['sd']])
subplot(3,3,3)
hist(history[:, 2])
subplot(3,3,4)
hist(history[:, 3])
subplot(3,3,5)
plot(history[:, slices['intercept']])
subplot(3,3,6)
plot(history[:, slices['sd']])
subplot(3,3,7)
plot(history[:, 2])
subplot(3,3,8)
plot(history[:, 3])
subplot(3,3,9)
plot(history[:, 4])

show()





'''
Created on Jan 27, 2010

@author: johnsalvatier
'''
import numpy 

def eigen(a, n = -1):
    
    if len(a.shape) == 0: # if we got a 0-dimensional array we have to turn it back into a 2 dimensional one 
        a = a[numpy.newaxis,numpy.newaxis]
        
    if n == -1:
        n = a.shape[0]
        
    eigenvalues, eigenvectors = numpy.linalg.eigh(a)

    indicies = numpy.argsort(eigenvalues)[::-1]
    return eigenvalues[indicies[0:n]], eigenvectors[:,indicies[0:n]]

def vectorsMult( matrix, vectors):
    """
    multiply a matrix by a list of vectors which should result in another list of vectors 
    the "list" axis should be the first axis
    """
    return numpy.dot(matrix, vectors.transpose()[numpy.newaxis,:,:])[:,0].transpose()


import model8

import pymc 
import multichain_mcmc
import numpy 
import cProfile 

def sample():
    sampler = multichain_mcmc.AmalaSampler(model8.model_gen)
    sampler.sample(ndraw = 500,  maxGradient = 1.3, mAccept = True, mConvergence = True)
    

cProfile.run("sample()", "amala.profile")


from numpy import *
import pymc
from scipy import stats
from scipy.stats import distributions as d

n = 100 

x = random.normal(scale = .2, size = n )
def model_gen():
    
    variables = []

    mu = pymc.Normal("mu", mu = 0, tau = 5**-2 )
    
    variables.append(mu)

    
    obs = pymc.Normal ("obs", mu = mu, tau = .2**-2, observed = True, value = x)

    return variables


'''
Created on Nov 25, 2009

@author: johnsalvatier
'''

from numpy import *
import pymc
from scipy import stats
import pylab 
ReData = arange(200, 3000, 25)
measured = 10.2 * (ReData )** .5 + stats.distributions.norm(mu = 0, scale = 55).rvs(len(ReData))
#pylab.plot(ReData, measured)
#pylab.plot(ReData, 1.59e-2 * (ReData )** 1.394,'r' )
#pylab.show()

def model():
    
    varlist = []
    
    sd =pymc.Uniform('sd', lower = 5, upper = 100) #pymc.Gamma("sd", 60 , beta =  2.0)
    varlist.append(sd)
    
    
    

    
    
    a = pymc.Uniform('a', lower = 0, upper = 100)#pymc.Normal('a', mu =  10, tau = 5**-2)
    b = pymc.Uniform('b', lower = .05, upper = 2.0)
    varlist.append(a)
    varlist.append(b)

    nonlinear = a  * (ReData )** b
    precision = sd **-2

    results = pymc.Normal('results', mu = nonlinear, tau = precision, value = measured, observed = True)
    varlist.append(results)
    
    return varlist


'''
Created on Nov 24, 2009

@author: johnsalvatier
'''
import model3

import pymc 
import multichain_mcmc
from pylab import *

sampler = multichain_mcmc.DreamSampler(model3.model_gen)
sampler.sample(nChains = 5, ndraw = 500, mConvergence = True,mAccept = True)


print sampler.R
history = sampler.history
print history.shape
samples = history.shape[0]
print sampler.acceptRatio
print sampler.burnIn

subplot(3,4,1)
hist(history[:, 0])
subplot(3,4,2)
hist(history[:, 1])
subplot(3,4,3)
hist(history[:, 2])
subplot(3,4,4)
hist(history[:, 3])
subplot(3,4,5)
plot(history[:, 0])
subplot(3,4,6)
plot(history[:, 1])
subplot(3,4,7)
plot(history[:, 2])
subplot(3,4,8)
plot(history[:, 3])
subplot(3,4,9)
plot(history[:,1],history[:, 2],'.')
subplot(3,4,10)
plot(history[:,2],history[:, 3],'.')
subplot(3,4,11)
plot(history[:,1],history[:, 3],'.')
subplot(3,4,12)
plot(history[:,0],history[:, 3],'.')



show()



'''
Created on Nov 24, 2009

@author: johnsalvatier
'''
import model2

import pymc 
import multichain_mcmc
from pylab import *

sampler = multichain_mcmc.AmalaSampler(model2.model)
sampler.sample(nChains = 5,ndraw = 500,samplesPerAdapatationParameter = .5,adaptationDecayLength = 100,  thin = 5)


print sampler.R
history = sampler.history
slices = sampler.slices
print history.shape
samples = history.shape[0]
print sampler.accepts_ratio
print sampler.burnIn
print sampler.time 

subplot(3,2,1)
hist(history[:, slices['mean']])
subplot(3,2,2)
hist(history[:, slices['sd']])

subplot(3,2,4)
plot(history[:, slices['mean']])
subplot(3,2,5)
plot(history[:, slices['sd']])




show()



'''
Created on Nov 24, 2009

@author: johnsalvatier
'''
import model2

import pymc 
import multichain_mcmc
from pylab import *

sampler = multichain_mcmc.DreamSampler(model2.model_gen)
sampler.sample(nChains = 5, ndraw = 1000)


print sampler.R
history = sampler.history
print history.shape
samples = history.shape[0]
print sampler.accepts_ratio
print sampler.burnIn
print sampler.time 

subplot(3,2,1)
hist(history[:, 0])
subplot(3,2,2)
hist(history[:, 1])
subplot(3,2,3)
hist(history[:, 2])
subplot(3,2,4)
plot(history[:, 0])
subplot(3,2,5)
plot(history[:, 1])
subplot(3,2,6)
plot(history[:,1],history[:, 2],'.')



show()



'''
Created on Nov 24, 2009

@author: johnsalvatier
'''
import model5

import pymc 
import multichain_mcmc
from pylab import *
import numpy 

sampler = multichain_mcmc.DreamSampler(model5.model_gen)
sampler.sample( mConvergence = True,mAccept = True)


print sampler.R
history = sampler.history
print history.shape
samples = history.shape[0]
print sampler.acceptRatio
print sampler.burnIn
print "time ", str(sampler.time)

subplot(3,3,1)
hist(history[:, 0])
subplot(3,3,2)
hist(history[:, 1])
subplot(3,3,3)
hist(history[:, 2])
subplot(3,3,3)
hist(history[:, 3])
subplot(3,3,5)
plot(history[:, 0])
subplot(3,3,6)
plot(history[:, 1])
subplot(3,3,8)
plot(history[:, 2])
subplot(3,3,9)
plot(history[:, 3])



show()



from numpy import *
import pymc
from scipy import stats
from scipy.stats import distributions as d

#parameters about the da
dimensions = 20
observations = 100
shape = (dimensions, observations)


# first generate some fake data
#generate a factor (just one)
trueFactorMagnitudes = d.norm(  loc = 0, scale = 1).rvs(observations)
# make up a factor loading
trueFactorLoadings = d.norm( loc = 1, scale = .2).rvs (dimensions)
#make up some error scales
trueErrorSds = d.gamma.rvs(5, scale = .05, size = dimensions)



#make up the actual data
data = (trueFactorMagnitudes[newaxis, :] * trueFactorLoadings[:, newaxis] + d.norm(loc = 0, scale = trueErrorSds[:, newaxis]).rvs(shape)).ravel()

def model_gen():
    
    variables = []
    
    factors = pymc.Normal("factormagnitudes",mu = zeros(observations), tau = ones(observations), )
    limits = ones(dimensions) * -Inf
    limits[0] = 0.0
    loadings = pymc.TruncatedNormal("factorloadings",mu = ones(dimensions), tau = ones(dimensions)*(1 **-2), a = limits, b = Inf)
    returnSDs = pymc.Gamma("residualsds", alpha = ones(dimensions) * 1 , beta = ones(dimensions) * .5)
    
    variables.append(loadings)
    variables.append (returnSDs)
    variables.append(factors)
    
    @pymc.deterministic
    def returnPrecisions ( stdev = returnSDs):
        precisions = (ones(shape) * (stdev**-2)[:, newaxis]).ravel()
    
        return precisions
    
    
    @pymc.deterministic
    def meanReturns (factors = factors, loadings = loadings):
    
        means = factors[newaxis, :] * loadings[:,newaxis]
    
        return means.ravel()
    
    returns = pymc.Normal ("returns", mu = meanReturns, tau = returnPrecisions, observed = True, value = data.ravel())
    
    variables.append(returns)
    
    return variables


'''
Created on Nov 24, 2009

@author: johnsalvatier
'''
import model1

import pymc 
import multichain_mcmc
from pylab import *

sampler = multichain_mcmc.AmalaSampler(model1.model)
sampler.sample(nChains = 5, ndraw = 500,  maxGradient = 100)


print sampler.R
history = sampler.history
slices = sampler.slices
print history.shape
samples = history.shape[0]
print sampler.accepts_ratio
print sampler.burnIn
print sampler.time 

subplot(3,3,1)
hist(history[:, slices['a']])
subplot(3,3,2)
hist(history[:, slices['b']])
subplot(3,3,3)
hist(history[:, slices['sd']])

subplot(3,3,4)
plot(history[:, slices['a']])
subplot(3,3,5)
plot(history[:, slices['b']])
subplot(3,3,6)
plot(history[:, slices['sd']])

subplot(3,3,7)
plot(history[:, slices['a']], history[:, slices['b']], '.')

show()



'''
Created on Nov 24, 2009

@author: johnsalvatier
'''
import model

import pymc 
import multichain_mcmc
from pylab import *

sampler = multichain_mcmc.DreamSampler(model.model)
sampler.sample()


print sampler.iter 
history = sampler.history
samples = history.shape[0]
print samples

subplot(2,1,1)
hist(history[:, 0])
subplot(2,1,2)
hist(history[:, 1])
show()



'''
Created on Jan 27, 2010

@author: johnsalvatier
'''
import numpy as np
import pymc


class SimulationHistory(object):
    
    group_indicies = {'all' : slice(None, None)}
    
    def __init__(self, maxChainDraws, nChains, dimensions):
        self._combined_history = np.zeros((nChains * maxChainDraws,dimensions ))
        self._sequence_histories = np.zeros((nChains, dimensions, maxChainDraws))
        self._logPSequences = np.zeros((nChains, maxChainDraws))
        self._logPHistory = np.zeros(nChains * maxChainDraws)
        
        self._sampling_start = np.Inf
        
        self._nChains = nChains
        self._dimensions = dimensions
        self.relevantHistoryStart = 0
        self.relevantHistoryEnd = 0
        
    
    def add_group(self, name, slices):
        indexes = range(self._dimensions)
        indicies = []
        for s in slices:
            indicies.extend(indexes[s])
            
        self.group_indicies[name] = np.array(indicies) 
            
    
    def record(self, vectors, logPs ,increment):
        if len(vectors.shape) < 3:
            self._record(vectors, logPs, increment)
        else:
            for i in range(vectors.shape[2]):
                self._record(vectors[:,:,i], logPs[:, i], increment)

    def _record(self, vectors, logPs, increment):
        self._sequence_histories[:,:,self.relevantHistoryEnd] = vectors
        self._combined_history[(self.relevantHistoryEnd *self._nChains) :(self.relevantHistoryEnd *self._nChains + self._nChains),:] = vectors
        self._logPSequences[:, self.relevantHistoryEnd] = logPs
        self._logPHistory[(self.relevantHistoryEnd *self._nChains) :(self.relevantHistoryEnd *self._nChains + self._nChains)] = logPs
        
        self.relevantHistoryEnd += 1
        self.relevantHistoryStart += increment
        

        
    def start_sampling(self):
        self._sampling_start = self.relevantHistoryEnd
        
    @property 
    def sequence_histories(self):
        return self.group_sequence_histories('all')
    
    def group_sequence_histories(self, name):
        return self._sequence_histories[:,self.group_indicies[name], np.ceil(self.relevantHistoryStart):self.relevantHistoryEnd]
    @property
    def nsequence_histories(self):
        return self.sequence_histories.shape[2]
    
    @property
    def combined_history(self):
        return self.group_combined_history('all')

    def group_combined_history(self, name):
        return self._combined_history[(np.ceil(self.relevantHistoryStart) *self._nChains):(self.relevantHistoryEnd * self._nChains),self.group_indicies[name]]

    
    @property
    def ncombined_history(self):
        return self.combined_history.shape[0]
    
    @property
    def samples(self):
        return self.group_samples('all')
    
    def group_samples(self, name):
        if self._sampling_start < np.Inf:
        
            start =  (max(np.ceil(self.relevantHistoryStart), self._sampling_start) *self._nChains)
            end = (self.relevantHistoryEnd * self._nChains)
        else:
            start = 0
            end = 0
        return self._combined_history[start:end,self.group_indicies[name]]
                
    @property
    def nsamples(self):
        return self.samples.shape[0]
    
    @property
    def combined_history_logps(self):
        return self._logPHistory[(np.ceil(self.relevantHistoryStart) *self._nChains):(self.relevantHistoryEnd * self._nChains)]
    
    
    "complete histories"
    
    @property
    def complete_combined_history(self):
        return self.group_complete_combined_history('all')
    
    def group_complete_sequence_histories(self, name):
        return self._sequence_histories[:,self.group_indicies[name], 0:self.relevantHistoryEnd]   
    
    @property
    def complete_sequence_histories(self):
        return self.group_complete_sequence_histories('all')
    
    @property
    def ncomplete_sequence_histories(self):
        return self.complete_sequence_histories.shape[2]
        
    def group_complete_combined_history(self, name):
        return self._combined_history[0:(self.relevantHistoryEnd * self._nChains),self.group_indicies[name]]


from __future__ import division
from numpy import *
import numpy 
import math
from utilities import *

'''
Created on Jan 11, 2010

@author: johnsalvatier
'''

class GRConvergence:
    """
    Gelman Rubin convergence diagnostic calculator class. It currently only calculates the naive
    version found in the first paper. It does not check to see whether the variances have been
    stabilizing so it may be misleading sometimes.
    """
    _R = numpy.Inf
    _V = numpy.Inf
    _VChange = numpy.Inf
    
    _W = numpy.Inf
    _WChange = numpy.Inf
    
    def __init__(self):
        pass
    
    def _get_R(self):
        return self._R
    
    R = property(_get_R)
    
    @property
    def VChange(self):
        return self._VChange
    
    @property
    def WChange(self):
        return self._WChange
    
    def update(self, history):
        """
        Updates the convergence diagnostic with the current history.
        """
        
        N = history.nsequence_histories
        
        sequences = history.sequence_histories

        variances  = numpy.var(sequences,axis = 2)

        means = numpy.mean(sequences, axis = 2)
        
        withinChainVariances = numpy.mean(variances, axis = 0)
        
        betweenChainVariances = numpy.var(means, axis = 0) * N
        
        varEstimate = (1 - 1.0/N) * withinChainVariances + (1.0/N) * betweenChainVariances

        self._R = numpy.sqrt(varEstimate/ withinChainVariances)
        
        self._WChange = numpy.abs(numpy.log(withinChainVariances /self._W)**.5)
        self._W = withinChainVariances
        
        self._VChange = numpy.abs(numpy.log(varEstimate /self._V)**.5)
        self._V = varEstimate
        
class CovarianceConvergence:
    """
    """
    relativeVariances = {}
    
    def update(self, history, group):
        
        relevantHistory = history.group_combined_history(group)


        self.relativeVariances[group] = self.rv(relevantHistory)
     
    @staticmethod   
    def rv(relevantHistory):  
        end = relevantHistory.shape[0]
        midpoint = floor(end/2)
        
        covariance1 = numpy.cov(relevantHistory[0:midpoint, :].transpose())
        covariance2 = numpy.cov(relevantHistory[midpoint:end, :].transpose())

        eigenvalues1, eigenvectors1 = eigen(covariance1)
        basis1 = (sqrt(eigenvalues1)[newaxis,:] * eigenvectors1)
        
        eigenvalues2, eigenvectors2 = eigen(covariance2)
        basis2 = (sqrt(eigenvalues2)[newaxis,:] * eigenvectors2)  
        
        # project the second basis onto the first basis
        projection = dot(linalg.inv(basis1), basis2)
        
        # find the releative size in each of the basis1 directions 
        return log(sum(projection**2, axis = 0)**.5) 
    
    @staticmethod
    def compare_cov(d, cov1, n1, cov2, n2):
        n = n1 + n2
        k = 2
        pooledWithinCov = (cov1*(n1 - 1) + cov2*(n2 -1))/(n - k)
        
        M = (n - k) * log (det(pooledWithinCov))  - (n1 - 1) * log(det(cov1)) - (n2 - 1) * log(det(cov2))
        h = 1 - (2 * d**2 + 3*d - 1)/(6 *(p+1)*(k-1)) * ( 1/(n1 - 1) + 1/(n2 -1)     - 1/(n - k))
        df = d * (d+1) *(k-1) /2
        
        return df, M * h


from __future__ import division
from numpy import *
import numpy 
from utilities import eigen


class AdaptedApproximation:
    
    location = None
    orientation = None
    
    
    transformation = None 
    basis = None

    A1 = 1e7
    e1 = 1e-5
    
    def __init__(self, initial_location, initial_orientation):
        
        self.location = initial_location
        self._orientation = initial_orientation
        
        self._n = shape(initial_orientation)[0]
        self._Ie = eye(self._n) * self.e1
        
        self._update_orientation(initial_orientation)
        
        

    def update(self, currentVectors, adaptation_rate):
        
        oldMean = self.location 
        oldOrientation = self._orientation
        
        self.location = project(oldMean + adaptation_rate *  mean(currentVectors - oldMean, axis = 0), self.A1)
        self._orientation = project(oldOrientation + adaptation_rate * (dot((currentVectors - oldMean).transpose(),currentVectors - oldMean)/self._n - oldOrientation), self.A1 )
        
        self._update_orientation(self._orientation)
        
    _last_calc = Inf
    def _update_orientation(self, orientation):
        orientation = orientation + self._Ie  
        
        if self._last_calc > 20:
            eigenvalues, eigenvectors = eigen(orientation)
              
            # find the basis that will be uncorrelated using the covariance matrix
            basis = (sqrt(eigenvalues)[newaxis,:] * eigenvectors).transpose()
            #find the matrix that will transform a vector into that basis
            
            self.basis = basis
            self.transformation = linalg.inv(basis )
            self._last_calc = 0
        else :
            self._last_calc += 1

        self.orientation = orientation
        
        
class AdaptedScale:
    
    scale = 1.0
    
    
    A1 = 1e7
    
    def __init__(self, optimal_acceptence, minimum_scaling):
        self._optimal_acceptence = optimal_acceptence
        self._minimum_scaling = minimum_scaling 
        
    def update(self, acceptance, adaptation_rate):
            
        self.scale = max(project( self.scale *( 1  + adaptation_rate * mean (acceptance - self._optimal_acceptence)), self.A1), self._minimum_scaling)
        
        
def project(x , A1):
    
    norm = sqrt(sum(x**2))
    if norm < A1:
        return x
    else:
        return x * A1 / norm


'''
Created on Nov 24, 2009

@author: johnsalvatier
'''
import model9

import pymc 
import multichain_mcmc
from pylab import *
from numpy import *
from cPickle import *

sampler = multichain_mcmc.AmalaSampler(model9.gen_model)
sampler.sample(nChains = 5, ndraw = 500,  maxGradient = 100)


print sampler.R
history = sampler.samples
slices = sampler.slices
print history.shape
samples = sampler.samples
print sampler.accepts_ratio
print sampler.burnIn
print sampler.time 


outfile = open('samples1.pkl', 'wb')
dump(samples, outfile)

outfile = open('slices.pkl', 'wb')
dump(slices, outfile)

for var, slice in slices.iteritems():
    print var, mean(samples[:,slice], axis = 0)
    

"""
subplot(2,3,1)
hist(history[:, slices['ad']])
subplot(2,3,2)
hist(history[:, slices['f1d']])
subplot(2,3,3)
hist(history[:, slices['logvar']])

subplot(2,3,4)
plot(history[:, slices['ad']])
subplot(2,3,5)
plot(history[:, slices['f1d']])
subplot(2,3,6)
plot(history[:, slices['logvar']])
"""


#show()



'''
Created on Jan 13, 2010

@author: johnsalvatier                 
'''
from __future__ import division
from pymc import *

from numpy import *
from convergence import GRConvergence, CovarianceConvergence
from adaptation import AdaptedApproximation, AdaptedScale
import scipy.optimize
from multichain import MultiChainSampler, MultiChain
import time 
import dream_components
from utilities import vectorsMult, eigen 
from simulation_history import *

class AmalaSampler(MultiChainSampler):
    """
    Implementation of Adaptive Metropolist adjusted Langevin algorithm (Adaptive MaLa) MCMC sampling technique.
    Requires PyMC branch with gradient information support in order to function:
        http://github.com/pymc-devs/pymc/tree/gradientBranch
    
    MaLa uses gradient information in order to bias the proposal distribution in the direction of increasing 
    probability density. Adaptive technique based on (1). Multiple chains are used in order to attempt to 
    improve convergence monitoring and adaptation. Resampling from the past (2) is used in order to avoid
    getting stuck in low probability regions. However (2) shows that resampling from the past generally slows
    down sampling, so we try to do this mostly near the beginning of sampling. Convergence assessment is 
    based on a naive implementation of the Gelman-Rubin convergence statistics.
    
    (1) AtchadŽ, Y. (2006). An Adaptive Version for the Metropolis Adjusted Langevin Algorithm with a Truncated Drift. Methodology and Computing in Applied Probability. 8 (2), 235-254. 
    (2) AtchadŽ, Y. (2009). Resampling from the past to improve on MCMC algorithms. FAR EAST JOURNAL OF THEORETICAL STATISTICS. 27 (1), 81-100. 
       
    """
    optimalAcceptance = .574
    A1 = 1e7
    e1 = 1e-5
    e2 = 1e-5
    outliersFound = 0
    
    
    acceptRatio = 0.0
    
    def sample(self, ndraw = 1000, samplesPerAdapatationParameter = 3, adaptationDecayLength = 250, variables_of_interest = None, scaling = True,minScale = .1, maxGradient = 1.0, debug = Inf, ndraw_max = None , nChains = 5, burnIn = 1000, thin = 2, initial_point = None, convergenceCriteria = 1.1, monitor_convergence = True, monitor_acceptence = True):
        """
        Samples from a posterior distribution using DREAM.
        
        Parameters
        ----------
        ndraw : int 
            minimum number of draws from the sample distribution to be returned 
        ndraw_max : int 
            maximum number of draws from the sample distribution to be returned
        nChains : int 
            number of different chains to employ
        burnInSize : int
            number of iterations (meaning draws / nChains) to do before doing actual sampling.
            
        Returns
        -------
            None : None 
                sample sets 
                self.history which contains the combined draws for all the chains
                self.iter which is the total number of iterations 
                self.acceptRatio which is the acceptance ratio
                self.burnIn which is the number of burn in iterations done 
                self.R  which is the gelman rubin convergence diagnostic for each dimension
        """
        startTime = time.time()
        if ndraw_max is None:
            ndraw_max = 10 * ndraw
        
        maxChainDraws = floor(ndraw_max/nChains)        
        self._initChains(nChains, ndraw_max)   
        
        # get the starting log likelihood and position for each of the chains 
        #currentVectors = self._vectors

 
        #initialize the history arrays  
        history = SimulationHistory(maxChainDraws,self._nChains, self.dimensions)
        
        if variables_of_interest is not None:
            slices = []
            for var in variables_of_interest:
                slices.append(self.slices[var])
        else:
            slices = [slice(None,None)]
        history.add_group('interest', slices)
        
        seedChainDraws = int(ceil(self.dimensions* 2/nChains) * nChains)
        history.record(self.draw_from_priors(seedChainDraws), zeros((self._nChains, seedChainDraws)), 0)

        # initilize the convergence diagnostic object
        grConvergence = GRConvergence()
        covConvergence = CovarianceConvergence()

        
        #2)now loop through and sample 
        
        minDrawIters = ceil(ndraw / nChains)
        maxIters = ceil(ndraw_max /nChains)
        
        
        iter = 1
    
        proposalVectors = 0
        reverseJumpLogPs = 0
        jumpLogPs = 0
        proposalGradLogPs = 0
       
        lastRecalculation = 0
        
        
        # try to find some approximate modes for starting the chain 
        for i in range(self._nChains):
            
            x0 = self._chains[i].vector
            def logp(vector):
                self._chains[i].propose(vector)
                
                try:
                    return -self._chains[i].logp
                except ZeroProbability:
                
                    return 300e100
                    
            
            
            def grad_logp(vector):
                self._chains[i].propose(vector)
                gradientd = self._chains[i].logp_gradient
                gradient = empty(self.dimensions)
                
                for p, v in gradientd.iteritems():
                    gradient[self.slices[str(p)]] = -ravel(v)
                
                return gradient
            
            mode = scipy.optimize.fmin_ncg(logp, x0, grad_logp, disp = False)
            self._chains[i].propose(mode)
            try:
                pass
            except:
                self._chains[i].propose(x0)
                
        print self._vectors
        
        if not (initial_point is None):
             
            self._propose_initial_point(initial_point)
        
        currentVectors = self._vectors
        currentLogPs = self._logPs
        currentGradLogPs = self._gradLogPs     

        adaptationConstant = min(self._nChains/(self.dimensions * samplesPerAdapatationParameter), 1)

        adapted_approximation = AdaptedApproximation(mean(history.combined_history, axis = 0), cov(history.combined_history.transpose()))
        adapted_scale = AdaptedScale(self.optimalAcceptance, minScale)

        accepts_ratio_weighting = 1 - exp(-1.0/30) 
        adaptationDecay = 1.0/adaptationDecayLength

        # continue sampling if:
        # 1) we have not drawn enough samples to satisfy the minimum number of iterations
        # 2) or any of the dimensions have not converged 
        # 3) and we have not done more than the maximum number of iterations 
        while (( history.nsamples < ndraw or 
                    any(grConvergence.R > convergenceCriteria) or 
                    any(abs(covConvergence.relativeVariances['all']) > .5 ) or 
                    any(abs(covConvergence.relativeVariances['interest']) > .1)) and 
                history.ncomplete_sequence_histories < maxChainDraws - 1):
            
            if iter  == burnIn:
                history.start_sampling()

            dream_step = (random.randint(30) == 0)
            
            if dream_step:
                proposalVectors = dream_components.dream2_proposals(currentVectors, history, self.dimensions, self._nChains, 2, array([1]),.05, 1e-9)
                jumpLogPs = 0
            else:
                proposalVectors, jumpLogPs = self._amala_proposals(currentVectors, currentGradLogPs, adapted_approximation, adapted_scale, maxGradient, iter %debug == 0)
                
            self._propose(proposalVectors)
            proposalGradLogPs = self._gradLogPs
            proposalLogPs = self._logPs
           
            if dream_step:
                reverseJumpLogPs = 0
            else:
                reverseJumpLogPs = self._reverseJumpP(currentVectors, proposalVectors, proposalGradLogPs, adapted_approximation, adapted_scale, maxGradient)
                
            #apply the metrop decision to decide whether to accept or reject each chain proposal        
            decisions, acceptance = self._metropolis_hastings(currentLogPs,proposalLogPs, jumpLogPs, reverseJumpLogPs) 
                
                
            if iter % debug == 0 :
                print "c", currentVectors
                print "cp", currentLogPs
                print "cg", currentGradLogPs
                print "j", jumpLogPs
                
                print "p", proposalVectors
                print "pp", proposalLogPs
                print "pg", proposalGradLogPs  
                print "rj", reverseJumpLogPs  
                print "a", acceptance
            
            self._update_accepts_ratio(accepts_ratio_weighting, acceptance)
            
            if monitor_acceptence and iter % 20 == 0:
                print "accepts ratio: ", self.accepts_ratio, " adapted scale: ", adapted_scale.scale
            
            self._reject(decisions)
    
            #make the current vectors the previous vectors 
            previousVectors = currentVectors
            currentVectors = choose(decisions[:,newaxis], (currentVectors, proposalVectors))

            currentLogPs = choose(decisions, (currentLogPs, proposalLogPs))
            currentGradLogPs = choose(decisions[:, newaxis], (currentGradLogPs, proposalGradLogPs))
            
                    
            # we only want to recalculate convergence criteria when we are past the burn in period
            if history.nsamples > ndraw and iter > lastRecalculation * 1.1:

                lastRecalculation = iter
                grConvergence.update(history)
                covConvergence.update(history,'all')
                covConvergence.update(history, 'interest')
                
                
                if monitor_convergence:
                    print "GR mean: ", mean(grConvergence.R), "StdDev: ", std(grConvergence.R),"max: ", max(grConvergence.R),"argmax : ", argmax(grConvergence.R)
                    print covConvergence.relativeVariances['interest']


            if iter % thin == 0:
                history.record(currentVectors, currentLogPs, .5)
            
            adaptation_rate = adaptationConstant * exp(-history.nsamples * adaptationDecay)
            
            adapted_approximation.update(currentVectors, adaptation_rate)
            adapted_scale.update(acceptance, adaptation_rate)

            iter += 1

            
            
        
        #3) finalize
        
        # only make the second half of draws available because that's the only part used by the convergence diagnostic
        self.samples = history.samples
        self.history = history.complete_combined_history
        self.iter = iter
        self.burnIn = burnIn 
        self.time = time.time() - startTime
         
        self.R = grConvergence.R
        
        self._finalizeChains()
            
    def _amala_proposals(self, currentVectors, currentGradientLogPs, adapted_approximation, adapted_scale, maxGradient , debug):
        """
        generates and returns proposal vectors given the current states
        """
        scaledOrientation = adapted_approximation.orientation * adapted_scale.scale**2 
        tGrad, shrink = self._truncate(currentVectors, currentGradientLogPs,adapted_approximation, maxGradient)
        drift = vectorsMult(scaledOrientation, tGrad /2)
        if debug:
            print "tGrad", tGrad
            print "orientation ", scaledOrientation
            print "mean p ", currentVectors + drift

        s = random.multivariate_normal(mean = zeros(self.dimensions) ,cov = scaledOrientation, size = self._nChains)
        
        proposalVectors = currentVectors + drift + s

        jumpLogPs = zeros(self._nChains)
        for i in range(self._nChains):
            jumpLogPs[i] = pymc.distributions.mv_normal_cov_like(x = s[i,:], mu = zeros(self.dimensions), C = scaledOrientation )
            
        return proposalVectors, jumpLogPs

    
    def _reverseJumpP(self, currentVectors, proposalVectors, proposalGradLogPs, adapted_approximation, adapted_scale, maxGradient):
        
        scaledOrientation = adapted_approximation.orientation * adapted_scale.scale**2 
        tGrad,shrink = self._truncate(currentVectors, proposalGradLogPs,adapted_approximation, maxGradient)
        drift = vectorsMult(scaledOrientation, tGrad /2)
        
        reverseJump = currentVectors - proposalVectors 
        reverseJumpS = reverseJump - drift
        

        reverseJumpLogPs = zeros(self._nChains)
        for i in range(self._nChains):
            reverseJumpLogPs[i] = pymc.distributions.mv_normal_cov_like(x = reverseJumpS[i,:], mu = zeros(self.dimensions), C = scaledOrientation )
            
        return reverseJumpLogPs

    def _truncate(self,vectors, gradLogPs, adapted_approximation, maxGradient):
        
        transformedGradients = vectorsMult(adapted_approximation.basis, gradLogPs)
        transformedVectors = vectorsMult(adapted_approximation.transformation, vectors - adapted_approximation.location)

        # truncate by rescaling
        normalNorms =sum(transformedVectors**2, axis = 1)**.5
        gradientNorms = sum(transformedGradients**2, axis = 1)**.5

        truncation = maxGradient * normalNorms
        scalings =  truncation /maximum(truncation, gradientNorms)


        return scalings[:, newaxis] * gradLogPs, max(mean(scalings), .1)   


    def _vectorsMult(self, matrix, vectors):
        """
        multiply a matrix by a list of vectors which should result in another list of vectors 
        the "list" axis should be the first axis
        """
        return dot(matrix, vectors.transpose()[newaxis,:,:])[:,0].transpose()


import model8

import pymc 
import multichain_mcmc
import numpy 
import line_profiler
import time 

def sample():
    sampler = multichain_mcmc.AmalaSampler(model8.model_gen)
    sampler.sample(ndraw = 500,  maxGradient = 1.3, mAccept = True, mConvergence = True)
    
start = time.time()

    
    
profile = line_profiler.LineProfiler(pymc.Stochastic.gradient,pymc.Stochastic.grad_logp )
profile.runcall(sample)
profile.print_stats()
print  (time.time() - start)    


'''
Created on Oct 24, 2009

# Author: John Salvatier <jsalvati@u.washington.edu>, 2009.
'''
from distutils.core import setup
from distutils.extension import Extension
from Cython.Distutils import build_ext


DISTNAME            = 'multichain_mcmc'
DESCRIPTION         = "Multichain MCMC framework and algorithms"
LONG_DESCRIPTION    ="""
                    A simple framework based on PyMC for multichain MCMC algorithms. 
                    
                    Contains working implementations of:
                        * DREAM/DREAM_ZS sampler : multichain_mcmc.dream.DreamSampler
                        * Adaptive Metropolis Adjusted Langevin Algorithm (AMALA) sampler : multichain_mcmc.amala.AmalaSampler
                        
                    See the sampler classes for details. AMALA sampler requires PyMC branch with gradient information support to function.
                        http://github.com/pymc-devs/pymc/tree/gradientBranch
                    """
MAINTAINER          = 'John Salvatier'
MAINTAINER_EMAIL    = "jsalvati@u.washington.edu"
URL                 = "pypi.python.org/pypi/multichain_mcmc"
LICENSE             = "BSD"
VERSION             = "0.3"

classifiers =  ['Development Status :: 3 - Alpha',
                'Programming Language :: Python',
                'License :: OSI Approved :: BSD License',
                'Intended Audience :: Science/Research',
                'Topic :: Scientific/Engineering',
                'Topic :: Scientific/Engineering :: Mathematics',
                'Operating System :: OS Independent']

if __name__ == "__main__":

    setup(name = DISTNAME,
          version = VERSION,
        maintainer  = MAINTAINER,
        maintainer_email = MAINTAINER_EMAIL,
        description = DESCRIPTION,
        license = LICENSE,
        url = URL,
        long_description = LONG_DESCRIPTION,
        packages = ['multichain_mcmc'], 
        classifiers =classifiers,
        install_requires=['pymc', 'numpy','scipy', 'cython'],
        cmdclass = {'build_ext': build_ext},
        ext_modules = [Extension("multichain_mcmc.rand_no_replace", ["multichain_mcmc/rand_no_replace.pyx"])])




'''
Created on Jan 20, 2010

@author: johnsalvatier
'''

from numpy import *
from pymc import *
import pymc

a = ones((3,4))

j= NumpyDeterministics.sum_jacobian('a', a = a, axis = 0)

print j * 1

x = Normal('z', mu = ones((3,4)), tau = ones((3,4)))
y = pymc.sum(x, axis = 1)

print y



from numpy import *
import pymc
from scipy import stats
from scipy.stats import distributions as d

#parameters about the da
dimensions = 100
observations = 100
shape = (dimensions, observations)

data = d.norm(  loc = 0, scale = 1).rvs((dimensions, observations))
def model_gen():
    
    variables = []
    
    means = pymc.Normal("means",mu = zeros(dimensions), tau = ones(dimensions))

    sds = pymc.Gamma("sds", alpha = ones(dimensions) * 1 , beta = ones(dimensions) * 1)

    
    variables.append(means)
    variables.append (sds)
    
    @pymc.deterministic
    def precisions ( stdev = sds):
        precisions = (ones(shape) * (stdev**-2)[:, newaxis]).ravel()
    
        return precisions
    
    @pymc.deterministic
    def obsMeans (means = means):
        return (ones(shape) * means[:,newaxis] ).ravel()
    
    
    obs = pymc.Normal ("obs", mu = obsMeans, tau = precisions, observed = True, value = data.ravel())
    
    variables.append(obs)
    
    return variables


'''
Created on Jan 11, 2010

@author: johnsalvatier
'''
from __future__ import division
from pymc import *

from numpy import *
from  rand_no_replace import *
from convergence import GRConvergence
import time 


class MultiChainSampler:
    """
    """
    dimensions = None
    slices = None
    accepts_ratio = 0
    
    _model_generator = None
    _chains = []

    
    def __init__(self, model_generator):
        """
        Initializes the sampler.
        
        Parameters
        ----------
            model_generator : func()
                a parameterless function which returns a collection of Stochastics which make up the model. 
                Will be called multiple times, once for each chain. If the Stochastics returned are not identical 
                (for example, based on different data) and distinct (separately instanced) the sampler will have problems.  
        """
        self._model_generator = model_generator 
        self._vectorize()
    
    def _vectorize(self):
        """Compute the dimension of the sampling space and identify the slices
        belonging to each stochastic.
        """
        self.dimensions = 0
        self.slices = {}
        
        model = self._model_generator()
        for variable in model:
            
            if isinstance(variable,Stochastic) and not variable.observed:

                if isinstance(variable.value, np.matrix):
                    p_len = len(variable.value.A.ravel())
                elif isinstance(variable.value, np.ndarray):
                    p_len = len(variable.value.ravel())
                else:
                    p_len = 1
                    
                self.slices[str(variable)] = slice(self.dimensions, self.dimensions + p_len)
                self.dimensions += p_len
    
    @property
    def _gradLogPs(self):
        gradLogPs = zeros((self._nChains, self.dimensions))
        
        for i in range(self._nChains):
            logp_gradients = self._chains[i].logp_gradient
            
            for stochastic, logp_gradient in logp_gradients.iteritems():
                
                gradLogPs[i,self.slices[str(stochastic)]] = ravel(logp_gradient)

        return gradLogPs

    @property 
    def _nChains(self):
        return len(self._chains)
    
    @property
    def _vectors(self):

        vectors = zeros((self._nChains, self.dimensions))
        
        for i  in range(self._nChains): 
            vectors[i, :] = self._chains[i].vector   
        
        return vectors
    
    @property
    def _logPs (self):
        logPs = zeros(self._nChains)
        
        for i in range(self._nChains):
            try:
                logPs[i] = self._chains[i].logp
            except ZeroProbability:
                # if the variables are not valid a zero probability exception will be raised and we need to deal with that
                logPs[i] = -Inf
                
        return logPs

        
    
    def _propose(self, proposalVectors):
        
        for i in range(self._nChains):
            self._chains[i].propose(proposalVectors[i, :]) 
            
    def _metropolis_hastings(self, currentLogPs, proposalLogPs, jumpLogP = 0, reverseJumpLogP = 0):
        """
        makes a decision about whether the proposed vector should be accepted
        """
        logMetropHastRatio = (proposalLogPs - currentLogPs) + (reverseJumpLogP - jumpLogP)
        decision = log(random.uniform(size = self._nChains)) < logMetropHastRatio

        return decision, minimum(1, exp(logMetropHastRatio))
    
    def _reject(self, decisions):
        
        for i in range(self._nChains):
            if decisions[i] == False:
                self._chains[i].reject()

    def _initChains(self, nChains, ndraw_max):
        for i in range(nChains): 
 
            model = self._model_generator()
            chain = MultiChain(self, model)
            chain.sampleInit(ndraw_max)

            self._chains.append(chain)  
            
                
    def _finalizeChains(self):
        
        for i in range(self._nChains):
            self._chains[i].sampleFinalize()
            
            
    def _update_accepts_ratio(self, weighting, acceptances):
        self.accepts_ratio = weighting * mean(acceptances) + (1-weighting) * self.accepts_ratio
        
    def draw_from_priors(self, draws):
        model = self._model_generator()
        vectors =  zeros((self._nChains, self.dimensions, draws))
        for i in range(draws): 
            for j in range(self._nChains):
            
                #generate a vector drawn from the prior distributions
                for variable in model:
                    if isinstance(variable,Stochastic) and not variable.observed:
                        drawFromPrior = np.ravel(variable.random())
 
                        vectors[j,self.slices[str(variable)], i] = drawFromPrior
        return vectors

    def _propose_initial_point(self, initial_point):
        vector = zeros(self.dimensions)
        for var, value in initial_point.iteritems():
            
            vector[self.slices[var]] = ravel(value)
            
        self._chains[0].propose(vector)

class MultiChain(MCMC):
    """
    DreamChain uses a special step method DreamStepper for stepping. It also allows the object controlling it to control
    the looping, proposing and rejecting, which is not possible with other MCMC objects. This allows the chains to be 
    interdependent.
    
    The MCMC._loop() function has been split up into several functions, so that both DREAM and other step methods can be 
    use simultaneously but it is not currently set up to do so, so only DREAM will work right now.
    """
    
    multiChainStepper = None
    
    def __init__(self, container , input=None, variables = None, db='ram', name='MCMC', calc_deviance=True, **kwds):
        """Initialize an MCMC instance.

        :Parameters:
          - input : module, list, tuple, dictionary, set, object or nothing.
              Model definition, in terms of Stochastics, Deterministics, Potentials and Containers.
              If nothing, all nodes are collected from the base namespace.
          - db : string
              The name of the database backend that will store the values
              of the stochastics and deterministics sampled during the MCMC loop.
          - verbose : integer
              Level of output verbosity: 0=none, 1=low, 2=medium, 3=high
          - **kwds :
              Keywords arguments to be passed to the database instantiation method.
        """
        MCMC.__init__(self, input = input, db = db, name = name, calc_deviance = calc_deviance, **kwds)
            
        self.use_step_method(MultiChainStepper, input, container)
        
    
    def sampleInit(self, iter, burn=0, thin=1, tune_interval=1000, tune_throughout=True, save_interval=None, verbose=0):
        """
        sample(iter, burn, thin, tune_interval, tune_throughout, save_interval, verbose)

        Initialize traces, run sampling loop, clean up afterward. Calls _loop.

        :Parameters:
          - iter : int
            Total number of iterations to do
          - burn : int
            Variables will not be tallied until this many iterations are complete, default 0
          - thin : int
            Variables will be tallied at intervals of this many iterations, default 1
          - tune_interval : int
            Step methods will be tuned at intervals of this many iterations, default 1000
          - tune_throughout : boolean
            If true, tuning will continue after the burnin period (True); otherwise tuning
            will halt at the end of the burnin period.
          - save_interval : int or None
            If given, the model state will be saved at intervals of this many iterations
          - verbose : boolean
        """

                
        self.assign_step_methods()

        # find the step method we are now using so we can manipulate it
        for step_method in self.step_methods:
            if type(step_method) == MultiChainStepper:
                self.multiChainStepper = step_method
                


        if burn >= iter:
            raise ValueError, 'Burn interval must be smaller than specified number of iterations.'
        self._iter = int(iter)
        self._burn = int(burn)
        self._thin = int(thin)
        self._tune_interval = int(tune_interval)
        self._tune_throughout = tune_throughout
        self._save_interval = save_interval

        length = int(np.ceil((1.0*iter-burn)/thin))
        self.max_trace_length = length

        # Flags for tuning
        self._tuning = True
        self._tuned_count = 0

        # no longer call the base function because it calls _loop which we are avoiding 
        #Sampler.sample(self, iter, length, verbose)
        #
        
        """
        Draws iter samples from the posterior.
        """
        self._cur_trace_index=0
        self.max_trace_length = iter
        self._iter = iter
        if verbose>0:
            self.verbose = verbose
        self.seed()

        # Initialize database -> initialize traces.
        if length is None:
            length = iter
        self.db._initialize(self._funs_to_tally, length)

        # Put traces on objects
        for v in self._variables_to_tally:
            v.trace = self.db._traces[v.__name__]

        # Loop
        self._current_iter = 0
        
        
        # Set status flag
        self.status='running'

        # Record start time
        start = time.time()
        

    def initStep(self, _current_iter):
        self._current_iter = _current_iter
        
        # primarily taken from MCMC._loop()
        if self.status == 'paused':
            return 

        i = self._current_iter
        
        # Tune at interval
        if i and not (i % self._tune_interval) and self._tuning:
            self.tune()
        
        if i == self._burn:
            if self.verbose>0:
                print 'Burn-in interval complete'
            if not self._tune_throughout:
                if self.verbose > 0:
                    print 'Stopping tuning due to burn-in being complete.'
                self._tuning = False
        
        
        
        
    def continueStep(self):
        
        i = self._current_iter
        # Tell all the step methods except the DREAM StepMethod to take a step (dream step method has already stepped)
        for step_method in self.step_methods:
            
            if step_method != multiChainStepper  : 
                if self.verbose > 2:
                    print 'Step method %s stepping' % step_method._id
                # Step the step method
                step_method.step()
        
        if i % self._thin == 0 and i >= self._burn:
            self.tally()
        
        if self._save_interval is not None:
            if i % self._save_interval==0:
                self.save_state()
        
        if not i % 10000 and i and self.verbose > 0:
            per_step = (time.time() - start)/i
            remaining = self._iter - i
            time_left = remaining * per_step
        
            print "Iteration %i of %i (%i:%02d:%02d remaining)" % (i, self._iter, time_left/3600, (time_left%3600)/60, (time_left%60))
        
        if not i % 1000:
            self.commit()
        
        self._current_iter += 1    
    
    def propose(self, proposalVector):
        if self.verbose > 2:
            print 'Step method %s stepping' % step_method._id
        
        
        self.multiChainStepper.propose(proposalVector)
    
    @property
    def vector(self):
        return self.multiChainStepper.vector
    
    @property
    def logp(self):
        return self.multiChainStepper.logp_plus_loglike
    
    @property
    def logp_gradient(self):
        return self.multiChainStepper.logp_gradient
        
    def reject(self):
        self.multiChainStepper.reject()
        
    
    def sampleFinalize(self):
        self._finalize()

class MultiChainStepper(StepMethod):
    
    
    def __init__(self, stochastics, container, verbose = 0, tally = True):
        
        self._container = container
        
        # Initialize superclass
        StepMethod.__init__(self, stochastics, tally=tally)
        
        if verbose is not None:
            self.verbose = verbose
        else:
            self.verbose = stochastic.verbose 
           
    @property 
    def vector(self):
        vector = zeros(self._container.dimensions)
        
        for stochastic in self.stochastics:
            vector[self._container.slices[str(stochastic)]] = ravel(stochastic.value)
            
        return vector

    @property
    def names(self):
        names = [] 
        for stochastic in self.stochastics:
            names.append(str(stochastic))
        return names       

    def propose(self, proposalVector):
        # mostly from adaptive metropolist step method

        for stochastic in self.stochastics:
            proposedValue = proposalVector[self._container.slices[str(stochastic)]]
            if iterable(stochastic.value):
                proposedValue = np.reshape(proposedValue,np.shape(stochastic.value))

            stochastic.value = proposedValue
            
            
    def reject (self):
        for stochastic in self.stochastics:
            stochastic.revert()
            
            


    


#!/usr/bin/env python
import numpy
from numpy import *
from pymc import *


N = 12


n = 120


i_logvol = random.normal(size = (n, N))
f_logvol = random.normal(size = (n, 1))
data_shape = ones ((n -1,N))

def gen_model():
   
    mean_hyper_mean = Normal('mean_hyper_mean', mu = 0, tau = 1.0/3**2)
    mean_hyper_logvar = Normal('mean_hyper_logvar', mu = -.5, tau = 1.0/1.0**2)
    mean = Normal('mean', mu = mean_hyper_mean * ones(N)[newaxis,:], tau = 1.0/exp(mean_hyper_logvar))

    
    ad_hyper_mean = Normal('ad_hyper_mean', mu = .2 , tau = 1.0/1.0**2)
    ad_hyper_logvar = Normal('ad_hyper_logvar', mu = -.5 , tau = 1.0/1.0**2)
    ad = Normal('ad', mu = ad_hyper_mean * ones(N)[newaxis,:], tau = 1.0/exp(ad_hyper_logvar))
    

    f1d_hyper_mean = Normal('f1d_hyper_mean', mu = .5 , tau = 1.0/2.0**2)
    f1d_hyper_logvar = Normal('f1d_hyper_logvar', mu = -.5 , tau = 1.0/1.0**2)
    f1d = Normal('f1d', mu = f1d_hyper_mean * ones(N)[newaxis,:], tau = 1.0/exp(f1d_hyper_logvar))

    logvar_hyper_mean = Normal('logvar_hyper_mean', mu = -.5 , tau = 1.0/2.0**2)
    logvar_hyper_logvar = Normal('logvar_hyper_logvar', mu = -.5 , tau = 1.0/1.0**2)
    logvar = Normal('logvar', mu =  logvar_hyper_mean * zeros(N)[newaxis,:] , tau =  1.0/exp(logvar_hyper_logvar))
    
    print 1
    tau1 =  exp(logvar)
    print 2, tau1, data_shape, tau1.__array_priority__
    tau2 = 1.0/tau1
    print 3
    z = data_shape/exp(f1d_hyper_logvar)
    print 4
    tau = data_shape * tau2
    print 5
    
    volatilities = Normal('vols', mu = mean + ad * i_logvol[0:(n-1),:] + f1d * f_logvol[1:n,:] , tau = data_shape/exp(logvar) , observed = True, value = i_logvol[1:n,:])
    return [mean, mean_hyper_mean, mean_hyper_logvar,
            ad, ad_hyper_mean, ad_hyper_logvar, 
            f1d,f1d_hyper_mean, f1d_hyper_logvar, 
            logvar, logvar_hyper_mean, logvar_hyper_logvar]